@@ -34,7 +34,8 @@ from aiosmb.wintypes.dtyp.constrcuted_security.security_descriptor import SECURI
 from aiosmb.commons.smbcontainer import *
 from aiosmb.commons.connection.target import *
 
-from aiosmb.crypto.pure.AES.AESCCM import aesCCMEncrypt, aesCCMDecrypt
+from aiosmb.crypto.symmetric import aesCCMEncrypt, aesCCMDecrypt
+#from aiosmb.crypto.pure.AES.AESCCM import aesCCMEncrypt, aesCCMDecrypt
 #from aiosmb.crypto.AESCCM_dome import aesCCMEncrypt, aesCCMDecrypt
 from aiosmb.crypto.BASE import cipherMODE
 from aiosmb.crypto.from_impacket import KDF_CounterMode, AES_CMAC
@@ -177,14 +178,14 @@ class SMBConnection:
 		self.FileHandleTable = {}
 		
 		self.SequenceWindow = 0
-		self.MaxTransactSize = 0
-		self.MaxReadSize = 0
-		self.MaxWriteSize = 0
+		self.MaxTransactSize = self.target.MaxTransactSize if self.target.MaxTransactSize is not None else 0x100000
+		self.MaxReadSize = self.target.MaxReadSize if self.target.MaxReadSize is not None else 0x100000
+		self.MaxWriteSize = self.target.MaxWriteSize if self.target.MaxWriteSize is not None else 0x100000
 		self.ServerGuid = None
 		self.RequireSigning = False
 		self.ServerName = None
 		self.ClientGUID = GUID.random()
-		
+
 		#self.Dialect = 0
 		self.SupportsFileLeasing = False
 		self.SupportsMultiCredit = False
@@ -630,9 +631,9 @@ class SMBConnection:
 
 			logger.log(1, 'Server selected dialect: %s' % self.selected_dialect)
 					
-			self.MaxTransactSize = min(0x100000, rply.command.MaxTransactSize)
-			self.MaxReadSize = min(0x100000, rply.command.MaxReadSize)
-			self.MaxWriteSize = min(0x100000, rply.command.MaxWriteSize)
+			self.MaxTransactSize = min(self.MaxTransactSize, rply.command.MaxTransactSize)
+			self.MaxReadSize = min(self.MaxReadSize, rply.command.MaxReadSize)
+			self.MaxWriteSize = min(self.MaxWriteSize, rply.command.MaxWriteSize)
 			self.ServerGuid = rply.command.ServerGuid
 			self.SupportsMultiChannel = NegotiateCapabilities.MULTI_CHANNEL in rply.command.Capabilities
 			self.SupportsFileLeasing = NegotiateCapabilities.LEASING in rply.command.Capabilities
@@ -966,7 +967,7 @@ class SMBConnection:
 				raise SMBCreateAccessDenied()
 				
 			else:
-				raise SMBException('', rply.header.Status)
+				raise SMBException('%s' % rply.header.Status.name, rply.header.Status)
 		
 		except Exception as e:
 			if return_reply == True:
