@@ -40,12 +40,15 @@ class DCERPCTCPConnection:
 					data = await self.reader.readexactly(response_header['frag_len'] - 24)
 					msg_data += data
 					await self.in_queue.put((msg_data, None))
+				
+				except asyncio.CancelledError:
+					return
 				except Exception as e:
 					await self.in_queue.put((None, e))
 					return
 		
 		except asyncio.CancelledError:
-			#the SMB connection is terminating
+			#the connection is terminating
 			return
 			
 		except Exception as e:	
@@ -73,10 +76,11 @@ class DCERPCTCPConnection:
 	
 	@red
 	async def connect(self):
-		con = asyncio.open_connection(self.ip, self.port)
+		
 		
 		try:
-			self.reader, self.writer = await asyncio.wait_for(con, int(self.timeout))
+			con = asyncio.open_connection(self.ip, self.port)
+			self.reader, self.writer = await asyncio.wait_for(con, None)
 		except asyncio.TimeoutError:
 			logger.debug('[DCERPCTCPConnection] Connection timeout')
 			return None, SMBConnectionTimeoutException() 
@@ -230,7 +234,7 @@ class DCERPCTCPTransport:
 		try:
 			if self.is_proxy is False:
 				# the TCP call already buffers one messages per get
-				data, err = await asyncio.wait_for(self.connection.in_queue.get(), timeout = self.target.timeout)
+				data, err = await asyncio.wait_for(self.connection.in_queue.get(), timeout = None)
 				return data, err
 			
 			err = None
@@ -242,7 +246,7 @@ class DCERPCTCPTransport:
 						self.buffer = self.buffer[response_header['frag_len']:]
 						return msg_data, err
 
-				data, err = await asyncio.wait_for(self.connection.in_queue.get(), timeout = 10)
+				data, err = await asyncio.wait_for(self.connection.in_queue.get(), timeout = None)
 				if err is not None:
 					return None, err
 				
